@@ -50,9 +50,10 @@ G_BEGIN_DECLS
 #define CHAMPLAIN_MAP_SOURCE_FACTORY_GET_CLASS(obj) \
   (G_TYPE_INSTANCE_GET_CLASS ((obj), CHAMPLAIN_TYPE_MAP_SOURCE_FACTORY, ChamplainMapSourceFactoryClass))
 
+typedef struct _ChamplainMapSourceFactoryPrivate ChamplainMapSourceFactoryPrivate;
+
 typedef struct _ChamplainMapSourceFactory ChamplainMapSourceFactory;
 typedef struct _ChamplainMapSourceFactoryClass ChamplainMapSourceFactoryClass;
-typedef struct _ChamplainMapSourceFactoryPrivate ChamplainMapSourceFactoryPrivate;
 
 struct _ChamplainMapSourceFactory
 {
@@ -96,6 +97,12 @@ gboolean champlain_map_source_factory_register (ChamplainMapSourceFactory *facto
  * OpenStreetMap Osmarender
  */
 #define CHAMPLAIN_MAP_SOURCE_OSM_OSMARENDER "osm-osmarender"
+/**
+ * CHAMPLAIN_MAP_SOURCE_OSM_MAPQUEST:
+ *
+ * Mapquest OpenStreetMap
+ */
+#define CHAMPLAIN_MAP_SOURCE_OSM_MAPQUEST "osm-mapquest"
 /**
  * CHAMPLAIN_MAP_SOURCE_OSM_CYCLE_MAP:
  *
@@ -123,6 +130,7 @@ gboolean champlain_map_source_factory_register (ChamplainMapSourceFactory *facto
 #else
 #define CHAMPLAIN_MAP_SOURCE_OSM_MAPNIK "OpenStreetMap I"
 #define CHAMPLAIN_MAP_SOURCE_OSM_OSMARENDER "OpenStreetMap II"
+#define CHAMPLAIN_MAP_SOURCE_OSM_MAPQUEST "Mapquest OSM"
 #define CHAMPLAIN_MAP_SOURCE_OSM_CYCLE_MAP "OpenCycleMap"
 #define CHAMPLAIN_MAP_SOURCE_OSM_TRANSPORT_MAP "Public Transport"
 #define CHAMPLAIN_MAP_SOURCE_OAM "OpenAerialMap"
