@@ -162,6 +162,9 @@ struct _ChamplainViewPrivate
   gdouble longitude;
   gdouble latitude;
 
+  /* Timer to track time between viewport updates */
+  GTimer *update_viewport_timer;
+
   /* Hack to get smaller x,y coordinates as the clutter limit is G_MAXINT16 */
   ChamplainFloatPoint anchor;
 
@@ -199,8 +202,6 @@ struct _ChamplainViewPrivate
   /* Lines and shapes */
   ClutterActor *polygon_layer;  /* Contains the polygons */
 
-  guint update_cb_id;
-  gboolean perform_update;
   gint tiles_loading;
 };
 
@@ -251,8 +252,6 @@ static void update_scale (ChamplainView *view);
 static void view_load_visible_tiles (ChamplainView *view);
 static void view_position_tile (ChamplainView *view,
     ChamplainTile *tile);
-static void view_update_state (ChamplainView *view,
-    ChamplainTile *tile);
 static void view_update_anchor (ChamplainView *view,
     gint x,
     gint y);
@@ -260,9 +259,9 @@ static gboolean view_set_zoom_level_at (ChamplainView *view,
     gint zoom_level,
     gint x,
     gint y);
-static void tile_state_notify (GObject *gobject,
-    GParamSpec *pspec,
-    gpointer data);
+static void tile_state_notify (ChamplainTile *tile,
+    G_GNUC_UNUSED GParamSpec *pspec,
+    ChamplainView *view);
 static void view_update_polygons (ChamplainView *view);
 static gboolean finger_scroll_key_press_cb (ClutterActor *actor,
     ClutterKeyEvent *event,
@@ -271,10 +270,7 @@ static void champlain_view_go_to_with_duration (ChamplainView *view,
     gdouble latitude,
     gdouble longitude,
     guint duration);
-static gboolean perform_update_cb (ChamplainView *view);
 static gboolean fill_tile_cb (FillTileCallbackData *data);
-static void tile_destroyed_cb (GObject *gobject,
-    gpointer data);
 
 #define SCALE_HEIGHT  20
 #define SCALE_PADDING 10
@@ -339,14 +335,11 @@ update_viewport (ChamplainView *view,
   DEBUG_LOG ()
 
   ChamplainViewPrivate *priv = view->priv;
-  gfloat old_lat, old_lon;
 
   ChamplainFloatPoint old_anchor;
 
   old_anchor.x = priv->anchor.x;
   old_anchor.y = priv->anchor.y;
-  old_lon = priv->longitude;
-  old_lat = priv->latitude;
 
   view_update_anchor (view,
       x + priv->anchor.x + priv->viewport_size.width / 2.0,
@@ -378,11 +371,8 @@ update_viewport (ChamplainView *view,
   priv->longitude = viewport_get_current_longitude (priv);
   priv->latitude = viewport_get_current_latitude (priv);
 
-  if (fabs (priv->longitude - old_lon) > 0.001)
-    g_object_notify (G_OBJECT (view), "longitude");
-
-  if (fabs (priv->latitude - old_lat) > 0.001)
-    g_object_notify (G_OBJECT (view), "latitude");
+  g_object_notify (G_OBJECT (view), "longitude");
+  g_object_notify (G_OBJECT (view), "latitude");
 }
 
 
@@ -801,8 +791,6 @@ champlain_view_dispose (GObject *object)
   ChamplainView *view = CHAMPLAIN_VIEW (object);
   ChamplainViewPrivate *priv = view->priv;
 
-  g_source_remove (priv->update_cb_id);
-
   if (priv->factory != NULL)
     {
       g_object_unref (priv->factory);
@@ -868,6 +856,9 @@ champlain_view_dispose (GObject *object)
   if (priv->goto_context != NULL)
     champlain_view_stop_go_to (view);
 
+  if (priv->update_viewport_timer != NULL)
+    g_timer_destroy (priv->update_viewport_timer);
+
   G_OBJECT_CLASS (champlain_view_parent_class)->dispose (object);
 }
 
@@ -877,6 +868,10 @@ champlain_view_finalize (GObject *object)
 {
   DEBUG_LOG ()
 
+  ChamplainViewPrivate *priv = CHAMPLAIN_VIEW (object)->priv;
+
+  g_free (priv->license_text);
+
   G_OBJECT_CLASS (champlain_view_parent_class)->finalize (object);
 }
 
@@ -1277,7 +1272,7 @@ champlain_view_class_init (ChamplainViewClass *champlainViewClass)
   /**
    * ChamplainView::animation-completed:
    *
-   * The ::animation-completed signal is emitted when any animation in the view
+   * The #ChamplainView::animation-completed signal is emitted when any animation in the view
    * ends.  This is a detailed signal.  For example, if you want to be signaled
    * only for go-to animation, you should connect to
    * "animation-completed::go-to".
@@ -1574,8 +1569,8 @@ champlain_view_init (ChamplainView *view)
   priv->show_scale = FALSE;
   priv->scale_unit = CHAMPLAIN_UNIT_KM;
   priv->max_scale_width = 100;
-  priv->perform_update = TRUE;
   priv->tiles_loading = 0;
+  priv->update_viewport_timer = g_timer_new();
 
   /* Setup map layer */
   priv->map_layer = g_object_ref (clutter_group_new ());
@@ -1644,22 +1639,11 @@ champlain_view_init (ChamplainView *view)
   /* Setup license */
   create_license (view);
 
-  priv->update_cb_id = g_timeout_add (250, (GSourceFunc) perform_update_cb, view);
-
   priv->state = CHAMPLAIN_STATE_DONE;
   g_object_notify (G_OBJECT (view), "state");
 }
 
 
-static gboolean
-perform_update_cb (ChamplainView *view)
-{
-/*  DEBUG_LOG() */
-  view->priv->perform_update = TRUE;
-  return TRUE;
-}
-
-
 static void
 viewport_pos_changed_cb (G_GNUC_UNUSED GObject *gobject,
     G_GNUC_UNUSED GParamSpec *arg1,
@@ -1676,11 +1660,10 @@ viewport_pos_changed_cb (G_GNUC_UNUSED GObject *gobject,
 
   if (fabs (x - priv->viewport_size.x) > 100 ||
       fabs (y - priv->viewport_size.y) > 100 ||
-      priv->perform_update)
+      g_timer_elapsed (priv->update_viewport_timer, NULL) > 0.25)
     {
-      priv->perform_update = FALSE;
-
       update_viewport (view, x, y);
+      g_timer_start (priv->update_viewport_timer);
     }
 }
 
@@ -1743,9 +1726,27 @@ finger_scroll_button_press_cb (G_GNUC_UNUSED ClutterActor *actor,
   DEBUG_LOG ()
 
   ChamplainViewPrivate *priv = view->priv;
+  guint zoom_level = priv->zoom_level + 1;
 
-  if (priv->zoom_on_double_click && event->button == 1 && event->click_count == 2)
-    return view_set_zoom_level_at (view, priv->zoom_level + 1, event->x, event->y);
+  if (!ZOOM_LEVEL_OUT_OF_RANGE (priv, zoom_level) && priv->zoom_on_double_click &&
+      event->button == 1 && event->click_count == 2)
+    {
+      gdouble lon, lat;
+
+      champlain_view_get_coords_from_event (view, (ClutterEvent *) event, &lat, &lon);
+
+      champlain_view_stop_go_to (view);
+
+      priv->zoom_level = zoom_level;
+
+      resize_viewport (view);
+
+      champlain_view_center_on (view, lat, lon);
+
+      g_object_notify (G_OBJECT (view), "zoom-level");
+
+      return TRUE;
+    }
 
   return FALSE; /* Propagate the event */
 }
@@ -2075,7 +2076,7 @@ champlain_view_stop_go_to (ChamplainView *view)
 
   g_signal_emit_by_name (view, "animation-completed::go-to", NULL);
 
-  g_free (priv->goto_context);
+  g_slice_free (GoToContext, priv->goto_context);
   priv->goto_context = NULL;
 }
 
@@ -2128,7 +2129,7 @@ champlain_view_go_to_with_duration (ChamplainView *view,
 
   champlain_view_stop_go_to (view);
 
-  ctx = g_new0 (GoToContext, 1);
+  ctx = g_slice_new (GoToContext);
   ctx->from_latitude = priv->latitude;
   ctx->from_longitude = priv->longitude;
   ctx->to_latitude = latitude;
@@ -2358,8 +2359,8 @@ champlain_view_remove_layer (ChamplainView *view,
  * champlain_view_get_coords_from_event:
  * @view: a #ChamplainView
  * @event: a #ClutterEvent
- * @lat: a variable where to put the latitude of the event
- * @lon: a variable where to put the longitude of the event
+ * @lat: (out): a variable where to put the latitude of the event
+ * @lon: (out): a variable where to put the longitude of the event
  *
  * Gets coordinates from button-press-event and button-release-event signals.
  *
@@ -2370,8 +2371,8 @@ champlain_view_remove_layer (ChamplainView *view,
 gboolean
 champlain_view_get_coords_from_event (ChamplainView *view,
     ClutterEvent *event,
-    gdouble *latitude,
-    gdouble *longitude)
+    gdouble *lat,
+    gdouble *lon)
 {
   DEBUG_LOG ()
 
@@ -2421,7 +2422,7 @@ champlain_view_get_coords_from_event (ChamplainView *view,
       return FALSE;
     }
 
-  return champlain_view_get_coords_at (view, x, y, latitude, longitude);
+  return champlain_view_get_coords_at (view, x, y, lat, lon);
 }
 
 
@@ -2430,8 +2431,8 @@ champlain_view_get_coords_from_event (ChamplainView *view,
  * @view: a #ChamplainView
  * @x: the x position in the view
  * @y: the y position in the view
- * @lat: a variable where to put the latitude of the event
- * @lon: a variable where to put the longitude of the event
+ * @lat: (out): a variable where to put the latitude of the event
+ * @lon: (out): a variable where to put the longitude of the event
  *
  * Gets latitude and longitude for the given x, y position in
  * the view. Use if you get coordinates from GtkEvents for example.
@@ -2444,8 +2445,8 @@ gboolean
 champlain_view_get_coords_at (ChamplainView *view,
     guint x,
     guint y,
-    gdouble *latitude,
-    gdouble *longitude)
+    gdouble *lat,
+    gdouble *lon)
 {
   DEBUG_LOG ()
 
@@ -2459,11 +2460,11 @@ champlain_view_get_coords_at (ChamplainView *view,
   rel_x = x - actor_x;
   rel_y = y - actor_y;
 
-  if (latitude)
-    *latitude = viewport_get_latitude_at (priv,
+  if (lat)
+    *lat = viewport_get_latitude_at (priv,
           priv->viewport_size.y + rel_y + priv->anchor.y);
-  if (longitude)
-    *longitude = viewport_get_longitude_at (priv,
+  if (lon)
+    *lon = viewport_get_longitude_at (priv,
           priv->viewport_size.x + rel_x + priv->anchor.x);
 
   return TRUE;
@@ -2478,7 +2479,7 @@ view_load_visible_tiles (ChamplainView *view)
   ChamplainViewPrivate *priv = view->priv;
   ChamplainRectangle viewport = priv->viewport_size;
   gint size;
-  GList *children;
+  GList *children, *child;
   gint x_count, y_count, x_first, y_first, x_end, y_end, max_x_end, max_y_end;
   gboolean *tile_map;
   gint arm_size, arm_max, turn;
@@ -2521,13 +2522,13 @@ view_load_visible_tiles (ChamplainView *view)
 
   DEBUG ("Range %d, %d to %d, %d", x_first, y_first, x_end, y_end);
 
-  tile_map = g_new0 (gboolean, x_count * y_count);
+  tile_map = g_slice_alloc0 (sizeof (gboolean) * x_count * y_count);
 
   /* Get rid of old tiles first */
   children = clutter_container_get_children (CLUTTER_CONTAINER (priv->map_layer));
-  for (; children != NULL; children = g_list_next (children))
+  for (child = children; child != NULL; child = g_list_next (child))
     {
-      ChamplainTile *tile = CHAMPLAIN_TILE (children->data);
+      ChamplainTile *tile = CHAMPLAIN_TILE (child->data);
 
       gint tile_x = champlain_tile_get_x (tile);
       gint tile_y = champlain_tile_get_y (tile);
@@ -2536,7 +2537,11 @@ view_load_visible_tiles (ChamplainView *view)
       if (tile_x < x_first || tile_x >= x_end ||
           tile_y < y_first || tile_y >= y_end ||
           zoom_level != priv->zoom_level)
-        clutter_container_remove_actor (CLUTTER_CONTAINER (priv->map_layer), CLUTTER_ACTOR (tile));
+        {
+          // inform map source to terminate loading the tile
+          champlain_tile_set_state (tile, CHAMPLAIN_STATE_DONE);
+          clutter_container_remove_actor (CLUTTER_CONTAINER (priv->map_layer), CLUTTER_ACTOR (tile));
+        }
       else
         {
           tile_map[(tile_y - y_first) * x_count + (tile_x - x_first)] = TRUE;
@@ -2575,13 +2580,11 @@ view_load_visible_tiles (ChamplainView *view)
                  notify::state signal is connected  */
               champlain_tile_set_state (tile, CHAMPLAIN_STATE_LOADING);
 
-              data = g_new (FillTileCallbackData, 1);
+              data = g_slice_new (FillTileCallbackData);
               data->tile = tile;
               data->map_source = priv->map_source;
 
-              g_signal_connect (tile, "destroy", G_CALLBACK (tile_destroyed_cb), view);
-
-              g_object_add_weak_pointer (G_OBJECT (tile), (gpointer *) &data->tile);
+              g_object_ref (tile);
               g_object_ref (priv->map_source);
 
               /* set priority high, otherwise tiles will be loaded after panning is done */
@@ -2596,7 +2599,7 @@ view_load_visible_tiles (ChamplainView *view)
         arm_size++;
     }
 
-  g_free (tile_map);
+  g_slice_free1 (sizeof (gboolean) * x_count * y_count, tile_map);
 }
 
 
@@ -2608,14 +2611,11 @@ fill_tile_cb (FillTileCallbackData *data)
   ChamplainTile *tile = data->tile;
   ChamplainMapSource *map_source = data->map_source;
 
-  if (data->tile)
-    {
-      g_object_remove_weak_pointer (G_OBJECT (data->tile), (gpointer *) &data->tile);
-      champlain_map_source_fill_tile (map_source, tile);
-    }
+  champlain_map_source_fill_tile (map_source, tile);
 
-  g_free (data);
+  g_slice_free (FillTileCallbackData, data);
   g_object_unref (map_source);
+  g_object_unref (tile);
 
   return FALSE;
 }
@@ -2646,53 +2646,49 @@ view_position_tile (ChamplainView *view,
 }
 
 
+static void
+remove_all_tiles (ChamplainView *view)
+{
+  DEBUG_LOG ()
+
+  ChamplainViewPrivate *priv = view->priv;
+  GList *children, *child;
+
+  children = clutter_container_get_children (CLUTTER_CONTAINER (priv->map_layer));
+  for (child = children; child != NULL; child = g_list_next (child))
+    {
+      ChamplainTile *tile = CHAMPLAIN_TILE (child->data);
+
+      champlain_tile_set_state (tile, CHAMPLAIN_STATE_DONE);
+      clutter_container_remove_actor (CLUTTER_CONTAINER (priv->map_layer), CLUTTER_ACTOR (tile));
+    }
+  g_list_free (children);
+}
+
+
+/**
+ * champlain_view_reload_tiles:
+ * @view: a #ChamplainView
+ *
+ * Reloads all visible tiles.
+ *
+ * Since: 0.8
+ */
 void
 champlain_view_reload_tiles (ChamplainView *view)
 {
   DEBUG_LOG ()
 
-  clutter_group_remove_all (CLUTTER_GROUP (view->priv->map_layer));
+  remove_all_tiles (view);
 
   view_load_visible_tiles (view);
 }
 
 
 static void
-tile_destroyed_cb (GObject *gobject,
-    gpointer data)
-{
-  DEBUG_LOG ()
-
-  ChamplainView *view = CHAMPLAIN_VIEW (data);
-  ChamplainTile *tile = CHAMPLAIN_TILE (gobject);
-  ChamplainViewPrivate *priv = view->priv;
-
-  if (champlain_tile_get_state (tile) == CHAMPLAIN_STATE_LOADING ||
-      champlain_tile_get_state (tile) == CHAMPLAIN_STATE_LOADED)
-    {
-      priv->tiles_loading--;
-      if (priv->tiles_loading == 0)
-        {
-          priv->state = CHAMPLAIN_STATE_DONE;
-          g_object_notify (G_OBJECT (view), "state");
-        }
-    }
-}
-
-
-static void
-tile_state_notify (GObject *gobject,
+tile_state_notify (ChamplainTile *tile,
     G_GNUC_UNUSED GParamSpec *pspec,
-    gpointer data)
-{
-  DEBUG_LOG ()
-
-  view_update_state (CHAMPLAIN_VIEW (data), CHAMPLAIN_TILE (gobject));
-}
-
-
-static void
-view_update_state (ChamplainView *view, ChamplainTile *tile)
+    ChamplainView *view)
 {
   DEBUG_LOG ()
 
@@ -2710,7 +2706,8 @@ view_update_state (ChamplainView *view, ChamplainTile *tile)
     }
   else if (tile_state == CHAMPLAIN_STATE_DONE)
     {
-      priv->tiles_loading--;
+      if (priv->tiles_loading > 0)
+        priv->tiles_loading--;
       if (priv->tiles_loading == 0)
         {
           priv->state = CHAMPLAIN_STATE_DONE;
@@ -2762,7 +2759,7 @@ champlain_view_set_map_source (ChamplainView *view,
       g_object_notify (G_OBJECT (view), "zoom-level");
     }
 
-  clutter_group_remove_all (CLUTTER_GROUP (priv->map_layer));
+  remove_all_tiles (view);
 
   update_license (view);
   champlain_view_center_on (view, priv->latitude, priv->longitude);
@@ -3074,7 +3071,7 @@ champlain_view_ensure_visible (ChamplainView *view,
 /**
  * champlain_view_ensure_markers_visible:
  * @view: a #ChamplainView
- * @markers: a NULL terminated array of #ChamplainMarker elements
+ * @markers: (array zero-terminated=1): a NULL terminated array of #ChamplainMarker elements
  * @animate: a #gboolean
  *
  * Changes the map's zoom level and center to make sure those markers are
@@ -3249,7 +3246,7 @@ champlain_view_get_max_zoom_level (ChamplainView *view)
  *
  * Gets the view's current map source.
  *
- * Returns: the view's current map source. If you need to keep a reference to the
+ * Returns: (transfer none): the view's current map source. If you need to keep a reference to the
  * map source then you have to call #g_object_ref.
  *
  * Since: 0.4
